import { Component, OnInit, Input, Output, EventEmitter } from "@angular/core";

@Component({
  selector: 'angular-io-pager',
  templateUrl: './pager.component.html',
  styleUrls: ['./pager.component.css']
})
export class PagerComponent implements OnInit {
  @Input()
  pageGap = 10;
  @Input()
  id = 1;
  @Input()
  noOfPages = 10;
  @Input()
  public set totalRecords(value: number) {
    this._total = value;

    if (value > 0) {
      this.pagePreparation();
    }
  }

  public get totalRecords(): number {
    return this._total;
  }

  @Output()
  pageClick: EventEmitter<object> = new EventEmitter();

  // page view count selectors
  from = 1;
  to = 0;
  pageLength = 20;
  currentPage = 1;
  pages: number[] = [];
  pageNum: number[] = [];
  showLeftNav = false;
  showRightNav = false;
  totalPageLength = 0;
  leftNavString = "<<";
  rightNavString = ">>";
  lastPageMax = 0;
  lastPageMin = 0;
  _total = 0;

  constructor() {}

  ngOnInit() {
    for (let i = 1; i <= this.pageLength; i++) {
      this.pages.push(i * this.pageGap);
    }

    this.to = this.pages[0];
  }

  pagePreparation() {
    this.reCalculatePageNumbers();

    setTimeout(() => {
      this.setSelectedPage();
    }, 0);
  }

  reCalculatePageNumbers() {
    const divided = (this.totalRecords / this.pageGap).toString().split(".");
    const wholePageLen = parseInt(divided[0], 10);
    let partialPageLen = 0;
    if (divided.length === 2) {
      // has one more page with limited records
      partialPageLen = 1;
    }

    this.totalPageLength = wholePageLen + partialPageLen;
    this.pageNum.length = 0;
    this.populatePages(1);
  }

  populatePages(start: number) {
    let until = 1;
    this.pageNum.length = 0;

    for (
      let i = start;
      i <= this.totalPageLength && until <= this.noOfPages;
      i++
    ) {
      this.pageNum.push(i);
      until++;
    }

    this.currentPage = this.lastPageMin = Math.min(...this.pageNum);
    this.lastPageMax = Math.max(...this.pageNum);
    this.setPrevNextNav();
  }

  pageLengthChange() {
    this.reCalculatePageNumbers();
    this.setSelectedPage();
  }

  pageChange(e) {
    if (e.target.classList.contains("cmdLeft")) {
      this.populatePages(this.lastPageMin - this.noOfPages);
      this.setSelectedPage();
    } else if (e.target.classList.contains("cmdRight")) {
      this.populatePages(++this.lastPageMax);
      this.setSelectedPage();
    } else {
      this.setSelectedPage(e.target);
    }
  }

  setSelectedPage(el: HTMLElement = null) {
    if (el != null) {
      this.currentPage = parseInt(el.innerText, 10);
    }
    this.to = Math.min(this.currentPage * this.pageGap, this.totalRecords);
    this.from = this.currentPage * this.pageGap - this.pageGap + 1;

    setTimeout(() => {
      el = <HTMLElement>(
        (el == null
          ? document
              .getElementById("pager" + this.id)
              .querySelector(".pageElements")
          : el)
      );

      let closePageNum = el;
      while (
        (closePageNum = closePageNum.parentElement) &&
        !closePageNum.classList.contains("pageNumbers")
      ) {}

      Array.from(closePageNum.children).forEach(x =>
        x.classList.remove("selected")
      );

      el.classList.add("selected");
    }, 0);

    this.emitPageClick();
  }

  setPrevNextNav() {
    this.showLeftNav = this.showRightNav = false;
    if (this.currentPage > this.noOfPages) {
      // current page is out of maximum pages can be shown, it means it is not the first set of page numbers. show left
      this.showLeftNav = true;
    }

    // get whether page is not a multiple of PagesToShow
    const currentPageIsInBetween =
      (this.currentPage / this.noOfPages).toString().split(".")
        .length === 2;

    // get highest page number in the current page
    // formula used = current page divided with maxPagetoShow, if found remainder,
    // page might have upcoming page links, so finding largest page number in that page.
    // ex. if the base showing is 40 in the case of 10 as the page length.
    // now 40 is the highest page showing, if 42 then 50 is the highest page showing.

    const currHighestPageShowing =
      parseInt((this.currentPage / this.noOfPages).toString(), 10) *
        this.noOfPages +
      (currentPageIsInBetween ? this.noOfPages : 0);

    if (currHighestPageShowing < this.totalPageLength) {
      this.showRightNav = true;
    }
  }

  emitPageClick() {
    this.pageClick.emit({ from: this.from, to: this.to });
  }
}
